@@ -22,7 +22,7 @@ help(void)
 " --log-tcp-sequence		Log TCP sequence numbers.\n\n"
 " --log-tcp-options		Log TCP options.\n\n"
 " --log-ip-options		Log IP options.\n\n",
-NETFILTER_VERSION);
+IPTABLES_VERSION);
 }
 
 static struct option opts[] = {
@@ -66,9 +66,10 @@ static struct ip6t_log_names ip6t_log_names[]
 static u_int8_t
 parse_level(const char *level)
 {
-	unsigned int lev;
+	unsigned int lev = -1;
+	unsigned int set = 0;
 
-	if (string_to_number(level, 0, 7, lev) == -1) {
+	if (string_to_number(level, 0, 7, &lev) == -1) {
 		unsigned int i = 0;
 
 		for (i = 0;
@@ -76,7 +77,7 @@ parse_level(const char *level)
 		     i++) {
 			if (strncasecmp(level, ip6t_log_names[i].name,
 					strlen(level)) == 0) {
-				if (lev != -1)
+				if (set++)
 					exit_error(PARAMETER_PROBLEM,
 						   "log-level `%s' ambiguous",
 						   level);
@@ -84,7 +85,7 @@ parse_level(const char *level)
 			}
 		}
 
-		if (lev == -1)
+		if (!set)
 			exit_error(PARAMETER_PROBLEM,
 				   "log-level `%s' unknown", level);
 	}
@@ -113,7 +114,7 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 			exit_error(PARAMETER_PROBLEM,
 				   "Can't specify --log-level twice");
 
-		if (check_inverse(optarg, &invert))
+		if (check_inverse(optarg, &invert, NULL, 0))
 			exit_error(PARAMETER_PROBLEM,
 				   "Unexpected `!' after --log-level");
 
@@ -126,7 +127,7 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 			exit_error(PARAMETER_PROBLEM,
 				   "Can't specify --log-prefix twice");
 
-		if (check_inverse(optarg, &invert))
+		if (check_inverse(optarg, &invert, NULL, 0))
 			exit_error(PARAMETER_PROBLEM,
 				   "Unexpected `!' after --log-prefix");
 
@@ -226,7 +227,7 @@ save(const struct ip6t_ip6 *ip, const struct ip6t_entry_target *target)
 		= (const struct ip6t_log_info *)target->data;
 
 	if (strcmp(loginfo->prefix, "") != 0)
-		printf("--log-prefix %s ", loginfo->prefix);
+		printf("--log-prefix \"%s\" ", loginfo->prefix);
 
 	if (loginfo->level != LOG_DEFAULT_LEVEL)
 		printf("--log-level %u ", loginfo->level);
@@ -243,7 +244,7 @@ static
 struct ip6tables_target log
 = { NULL,
     "LOG",
-    NETFILTER_VERSION,
+    IPTABLES_VERSION,
     IP6T_ALIGN(sizeof(struct ip6t_log_info)),
     IP6T_ALIGN(sizeof(struct ip6t_log_info)),
     &help,
