@@ -43,7 +43,7 @@ static void help(void)
 	       " --ulog-cprange size		Bytes of each packet to be passed\n"
 	       " --ulog-qthreshold		Threshold of in-kernel queue\n"
 	       " --ulog-prefix prefix		Prefix log messages with this prefix.\n\n",
-	       NETFILTER_VERSION);
+	       IPTABLES_VERSION);
 }
 
 static struct option opts[] = {
@@ -87,7 +87,7 @@ static int parse(int c, char **argv, int invert, unsigned int *flags,
 			exit_error(PARAMETER_PROBLEM,
 				   "Can't specify --ulog-nlgroup twice");
 
-		if (check_inverse(optarg, &invert))
+		if (check_inverse(optarg, &invert, NULL, 0))
 			exit_error(PARAMETER_PROBLEM,
 				   "Unexpected `!' after --ulog-nlgroup");
 		group_d = atoi(optarg);
@@ -105,7 +105,7 @@ static int parse(int c, char **argv, int invert, unsigned int *flags,
 			exit_error(PARAMETER_PROBLEM,
 				   "Can't specify --ulog-prefix twice");
 
-		if (check_inverse(optarg, &invert))
+		if (check_inverse(optarg, &invert, NULL, 0))
 			exit_error(PARAMETER_PROBLEM,
 				   "Unexpected `!' after --ulog-prefix");
 
@@ -187,9 +187,10 @@ print(const struct ipt_ip *ip,
 	printf("queue_threshold %d ", loginfo->qthreshold);
 }
 
+static
 struct iptables_target ulog = { NULL,
 	"ULOG",
-	NETFILTER_VERSION,
+	IPTABLES_VERSION,
 	IPT_ALIGN(sizeof(struct ipt_ulog_info)),
 	IPT_ALIGN(sizeof(struct ipt_ulog_info)),
 	&help,
