@@ -96,7 +96,7 @@ help(void)
 "ICMP v%s options:\n"
 " --icmp-type [!] typename	match icmp type\n"
 "				(or numeric type or type/code)\n"
-"\n", NETFILTER_VERSION);
+"\n", IPTABLES_VERSION);
 	print_icmptypes();
 }
 
@@ -133,7 +133,7 @@ parse_icmp(const char *icmptype, u_int8_t *type, u_int8_t code[])
 	} else {
 		char *slash;
 		char buffer[strlen(icmptype) + 1];
-		int number;
+		unsigned int number;
 
 		strcpy(buffer, icmptype);
 		slash = strchr(buffer, '/');
@@ -141,14 +141,12 @@ parse_icmp(const char *icmptype, u_int8_t *type, u_int8_t code[])
 		if (slash)
 			*slash = '\0';
 
-		number = string_to_number(buffer, 0, 255);
-		if (number == -1)
+		if (string_to_number(buffer, 0, 255, &number) == -1)
 			exit_error(PARAMETER_PROBLEM,
 				   "Invalid ICMP type `%s'\n", buffer);
 		*type = number;
 		if (slash) {
-			number = string_to_number(slash+1, 0, 255);
-			if (number == -1)
+			if (string_to_number(slash+1, 0, 255, &number) == -1)
 				exit_error(PARAMETER_PROBLEM,
 					   "Invalid ICMP code `%s'\n",
 					   slash+1);
@@ -185,8 +183,7 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 
 	switch (c) {
 	case '1':
-		if (check_inverse(optarg, &invert))
-			optind++;
+		check_inverse(optarg, &invert, &optind, 0);
 		*nfcache |= parse_icmp(argv[optind-1],
 				       &icmpinfo->type,
 				       icmpinfo->code);
@@ -275,10 +272,11 @@ static void final_check(unsigned int flags)
 {
 }
 
+static
 struct iptables_match icmp
 = { NULL,
     "icmp",
-    NETFILTER_VERSION,
+    IPTABLES_VERSION,
     IPT_ALIGN(sizeof(struct ipt_icmp)),
     IPT_ALIGN(sizeof(struct ipt_icmp)),
     &help,
