@@ -16,7 +16,7 @@ help(void)
 "MARK match v%s options:\n"
 "[!] --mark value[/mask]         Match nfmark value with optional mask\n"
 "\n",
-NETFILTER_VERSION);
+IPTABLES_VERSION);
 }
 
 static struct option opts[] = {
@@ -45,8 +45,7 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 	switch (c) {
 		char *end;
 	case '1':
-		if (check_inverse(optarg, &invert))
-			optind++;
+		check_inverse(optarg, &invert, &optind, 0);
 		markinfo->mark = strtoul(optarg, &end, 0);
 		if (*end == '/') {
 			markinfo->mask = strtoul(end+1, &end, 0);
@@ -112,7 +111,7 @@ static
 struct iptables_match mark
 = { NULL,
     "mark",
-    NETFILTER_VERSION,
+    IPTABLES_VERSION,
     IPT_ALIGN(sizeof(struct ipt_mark_info)),
     IPT_ALIGN(sizeof(struct ipt_mark_info)),
     &help,
