@@ -21,7 +21,7 @@ help(void)
 "				match destination port(s)\n"
 " --ports port[,port:port,port]\n"
 "				match both source and destination port(s)\n",
-NETFILTER_VERSION);
+IPTABLES_VERSION);
 }
 
 static struct option opts[] = {
@@ -270,7 +270,7 @@ static void save(const struct ipt_ip *ip, const struct ipt_entry_match *match)
 
 	for (i=0; i < IPT_MULTI_PORTS; i++) {
                 if (pflags & (1<<i)
-                    && !(~minfo->ports[i]))
+                    && minfo->ports[i] == 65535)
                         break;
                 if (i == IPT_MULTI_PORTS-1
                     && minfo->ports[i-1] == minfo->ports[i])
@@ -279,7 +279,7 @@ static void save(const struct ipt_ip *ip, const struct ipt_entry_match *match)
 		print_port(minfo->ports[i], ip->proto, 0);
                 if (pflags & (1<<i)) {
                         printf(":");
-                        print_port(minfo->ports[i++], ip->proto, 0);
+                        print_port(minfo->ports[++i], ip->proto, 0);
                 }
 	}
 	printf(" ");
@@ -288,7 +288,7 @@ static void save(const struct ipt_ip *ip, const struct ipt_entry_match *match)
 struct iptables_match mport
 = { NULL,
     "mport",
-    NETFILTER_VERSION,
+    IPTABLES_VERSION,
     IPT_ALIGN(sizeof(struct ipt_mport)),
     IPT_ALIGN(sizeof(struct ipt_mport)),
     &help,
