@@ -24,7 +24,7 @@ help(void)
 " [!] --srcpool NAME|INDEX\n"
 " [!] --dstpool NAME|INDEX\n"
 "			Pool index (or name from %s) to match\n"
-"\n", NETFILTER_VERSION, IPPOOL_CONF);
+"\n", IPTABLES_VERSION, IPPOOL_CONF);
 }
 
 static struct option opts[] = {
@@ -59,13 +59,13 @@ parse(int c, char **argv, int invert, unsigned int *flags,
 
 	switch (c) {
 	case '1':
-		if (check_inverse(optarg, &invert)) optind++;
+		check_inverse(optarg, &invert, &optind, 0);
 		info->src = ip_pool_get_index(argv[optind-1]);
 		if (invert) info->flags |= IPT_POOL_INV_SRC;
 		*flags = 1;
 		break;
 	case '2':
-		if (check_inverse(optarg, &invert)) optind++;
+		check_inverse(optarg, &invert, &optind, 0);
 		info->dst = ip_pool_get_index(argv[optind-1]);
 		if (invert) info->flags |= IPT_POOL_INV_DST;
 		*flags = 1;
@@ -126,7 +126,7 @@ static
 struct iptables_match pool
 = { NULL,
     "pool",
-    NETFILTER_VERSION,
+    IPTABLES_VERSION,
     IPT_ALIGN(sizeof(struct ipt_pool_info)),
     IPT_ALIGN(sizeof(struct ipt_pool_info)),
     &help,
