@@ -5,7 +5,9 @@
  * Authors:
  * 	Harald Welte <laforge@gnumonks.org>
  * 	Rusty Russell <rusty@linuxcare.com.au>
+ * This code is distributed under the terms of GNU GPL v2
  *
+ * $Id: ip6tables-restore.c,v 1.10 2002/08/14 11:40:41 laforge Exp $
  */
 
 #include <getopt.h>
@@ -22,17 +24,13 @@
 #define DEBUGP(x, args...) 
 #endif
 
-extern int for_each_chain(int (*fn)(const ip6t_chainlabel, int, ip6tc_handle_t *), int verbose, int builtinstoo, ip6tc_handle_t *handle);
-extern int flush_entries(const ip6t_chainlabel chain, int verbose, ip6tc_handle_t *handle);
-extern int delete_chain(const ip6t_chainlabel chain, int verbose, ip6tc_handle_t *handle);
-
 static int binary = 0, counters = 0, verbose = 0, noflush = 0;
 
 /* Keeping track of external matches and targets.  */
 static struct option options[] = {
 	{ "binary", 0, 0, 'b' },
 	{ "counters", 0, 0, 'c' },
-/*	{ "verbose", 1, 0, 'v' }, */
+	{ "verbose", 1, 0, 'v' },
 	{ "help", 0, 0, 'h' },
 	{ "noflush", 0, 0, 'n'},
 	{ "modprobe", 1, 0, 'M'},
@@ -61,9 +59,9 @@ ip6tc_handle_t create_handle(const char *tablename, const char* modprobe)
 	handle = ip6tc_init(tablename);
 
 	if (!handle) {
-                /* try to insmod the module if iptc_init failed */
-                ip6tables_insmod("ip6_tables", modprobe);
-                handle = ip6tc_init(tablename);
+		/* try to insmod the module if iptc_init failed */
+		ip6tables_insmod("ip6_tables", modprobe);
+		handle = ip6tc_init(tablename);
 	}
 
 	if (!handle) {
@@ -79,6 +77,29 @@ int parse_counters(char *string, struct ip6t_counters *ctr)
 	return (sscanf(string, "[%llu:%llu]", &ctr->pcnt, &ctr->bcnt) == 2);
 }
 
+/* global new argv and argc */
+static char *newargv[255];
+static int newargc;
+
+/* function adding one argument to newargv, updating newargc 
+ * returns true if argument added, false otherwise */
+static int add_argv(char *what) {
+	DEBUGP("add_argv: %s\n", what);
+	if (what && ((newargc + 1) < sizeof(newargv)/sizeof(char *))) {
+		newargv[newargc] = strdup(what);
+		newargc++;
+		return 1;
+	} else 
+		return 0;
+}
+
+static void free_argv(void) {
+	int i;
+
+	for (i = 0; i < newargc; i++)
+		free(newargv[i]);
+}
+
 int main(int argc, char *argv[])
 {
 	ip6tc_handle_t handle;
@@ -86,12 +107,11 @@ int main(int argc, char *argv[])
 	unsigned int line = 0;
 	int c;
 	char curtable[IP6T_TABLE_MAXNAMELEN + 1];
-	char curchain[IP6T_FUNCTION_MAXNAMELEN + 1];
 	FILE *in;
 	const char *modprobe = 0;
 
 	program_name = "ip6tables-restore";
-	program_version = NETFILTER_VERSION;
+	program_version = IPTABLES_VERSION;
 
 #ifdef NO_SHARED_LIBS
 	init_extensions();
@@ -105,9 +125,12 @@ int main(int argc, char *argv[])
 			case 'c':
 				counters = 1;
 				break;
+			case 'v':
+				verbose = 1;
+				break;
 			case 'h':
 				print_usage("ip6tables-restore",
-					    NETFILTER_VERSION);
+					    IPTABLES_VERSION);
 				break;
 			case 'n':
 				noflush = 1;
@@ -131,17 +154,7 @@ int main(int argc, char *argv[])
 		exit(1);
 	}
 	else in = stdin;
-/*
-	handle = iptc_init("filter");
-	if (!handle)
-		exit_error(VERSION_PROBLEM,
-			   "can't initialize iptables-restore: %s",
-			   iptc_strerror(errno));
-
-	if (!clean_slate(&handle))
-		exit_error(OTHER_PROBLEM, "Deleting old chains: %s",
-			   iptc_strerror(errno));
-*/
+	
 	/* Grab standard input. */
 	while (fgets(buffer, sizeof(buffer), in)) {
 		int ret;
@@ -195,16 +208,15 @@ int main(int argc, char *argv[])
 					   program_name, line);
 				exit(1);
 			}
-			strncpy(curchain, chain, IP6T_FUNCTION_MAXNAMELEN);
 
-			/* why the f... does iptc_builtin not work here ? */
-			/* FIXME: abort if chain creation fails --RR */
-//			if (!iptc_builtin(curchain, &handle)) {
-				DEBUGP("Creating new chain '%s'\n", curchain);
-				if (!ip6tc_create_chain(curchain, &handle))
-				DEBUGP("unable to create chain '%s':%s\n", curchain,
-					strerror(errno));
-//			}
+			if (!ip6tc_builtin(chain, handle)) {
+				DEBUGP("Creating new chain '%s'\n", chain);
+				if (!ip6tc_create_chain(chain, &handle))
+					exit_error(PARAMETER_PROBLEM,
+						   "error creating chain "
+						   "'%s':%s\n", chain,
+						   strerror(errno));
+			}
 
 			policy = strtok(NULL, " \t\n");
 			DEBUGP("line %u, policy '%s'\n", line, policy);
@@ -244,66 +256,119 @@ int main(int argc, char *argv[])
 			ret = 1;
 
 		} else {
-			char *newargv[1024];
-			int i,a, argvsize;
+			int a;
 			char *ptr = buffer;
 			char *pcnt = NULL;
 			char *bcnt = NULL;
+			char *parsestart;
+
+			/* the parser */
+			char *param_start, *curchar;
+			int quote_open;
+
+			/* reset the newargv */
+			newargc = 0;
 
 			if (buffer[0] == '[') {
+				/* we have counters in our input */
 				ptr = strchr(buffer, ']');
 				if (!ptr)
 					exit_error(PARAMETER_PROBLEM,
 						   "Bad line %u: need ]\n",
 						   line);
+
 				pcnt = strtok(buffer+1, ":");
+				if (!pcnt)
+					exit_error(PARAMETER_PROBLEM,
+						   "Bad line %u: need :\n",
+						   line);
+
 				bcnt = strtok(NULL, "]");
-			} 
+				if (!bcnt)
+					exit_error(PARAMETER_PROBLEM,
+						   "Bad line %u: need ]\n",
+						   line);
 
-			newargv[0] = argv[0];
-			newargv[1] = "-t";
-			newargv[2] = (char *) &curtable;
-			newargv[3] = "-A";
-			newargv[4] = (char *) &curchain;
-			argvsize = 5;
+				/* start command parsing after counter */
+				parsestart = ptr + 1;
+			} else {
+				/* start command parsing at start of line */
+				parsestart = buffer;
+			}
 
+			add_argv(argv[0]);
+			add_argv("-t");
+			add_argv((char *) &curtable);
+			
 			if (counters && pcnt && bcnt) {
-				newargv[5] = "--set-counters";
-				newargv[6] = (char *) pcnt;
-				newargv[7] = (char *) bcnt;
-				argvsize = 8;
-			}
-				
-			// strtok initcialize!
-			if ( buffer[0]!='[' )
-			{
-				if (!(newargv[argvsize] = strtok(buffer, " \t\n")))
-					goto ImLaMeR;
-					//break;
-				argvsize++;
+				add_argv("--set-counters");
+				add_argv((char *) pcnt);
+				add_argv((char *) bcnt);
 			}
 
-			/* strtok: a function only a coder could love */
-			for (i = argvsize; i < sizeof(newargv)/sizeof(char *); 
-					i++) {
-				if (!(newargv[i] = strtok(NULL, " \t\n")))
-					break;
-				ptr = NULL;
-			}
-ImLaMeR:		if (i == sizeof(newargv)/sizeof(char *)) {
-				fprintf(stderr,
-					"%s: line %u too many arguments\n",
-					program_name, line);
-				exit(1);
+			/* After fighting with strtok enough, here's now
+			 * a 'real' parser. According to Rusty I'm now no
+			 * longer a real hacker, but I can live with that */
+
+			quote_open = 0;
+			param_start = parsestart;
+			
+			for (curchar = parsestart; *curchar; curchar++) {
+				if (*curchar == '"') {
+					if (quote_open) {
+						quote_open = 0;
+						*curchar = ' ';
+					} else {
+						quote_open = 1;
+						param_start++;
+					}
+				} 
+				if (*curchar == ' '
+				    || *curchar == '\t'
+				    || * curchar == '\n') {
+					char param_buffer[1024];
+					int param_len = curchar-param_start;
+
+					if (quote_open)
+						continue;
+
+					if (!param_len) {
+						/* two spaces? */
+						param_start++;
+						continue;
+					}
+					
+					/* end of one parameter */
+					strncpy(param_buffer, param_start,
+						param_len);
+					*(param_buffer+param_len) = '\0';
+
+					/* check if table name specified */
+					if (!strncmp(param_buffer, "-t", 3)
+                                            || !strncmp(param_buffer, "--table", 8)) {
+						exit_error(PARAMETER_PROBLEM, 
+						   "Line %u seems to have a "
+						   "-t table option.\n", line);
+						exit(1);
+					}
+
+					add_argv(param_buffer);
+					param_start += param_len + 1;
+				} else {
+					/* regular character, skip */
+				}
 			}
 
-			DEBUGP("===>calling do_command6(%u, argv, &%s, handle):\n",
-					i, curtable);
+			DEBUGP("calling do_command6(%u, argv, &%s, handle):\n",
+				newargc, curtable);
 
-			for (a = 0; a <= i; a++)
+			for (a = 0; a < newargc; a++)
 				DEBUGP("argv[%u]: %s\n", a, newargv[a]);
 
-			ret = do_command6(i, newargv, &newargv[2], &handle);
+			ret = do_command6(newargc, newargv, 
+					 &newargv[2], &handle);
+
+			free_argv();
 		}
 		if (!ret) {
 			fprintf(stderr, "%s: line %u failed\n",
