@@ -3,6 +3,13 @@
 /*
  * Author: Paul.Russell@rustcorp.com.au and mneuling@radlogic.com.au
  *
+ * (C) 2000-2002 by the netfilter coreteam <coreteam@netfilter.org>:
+ * 		    Paul 'Rusty' Russell <rusty@rustcorp.com.au>
+ * 		    Marc Boucher <marc+nf@mbsi.ca>
+ * 		    James Morris <jmorris@intercode.com.au>
+ * 		    Harald Welte <laforge@gnumonks.org>
+ * 		    Jozsef Kadlecsik <kadlec@blackhole.kfki.hu>
+ *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
  *	the Free Software Foundation; either version 2 of the License, or
@@ -82,7 +89,7 @@
 #define CMD_RENAME_CHAIN	0x1000U
 #define NUMBER_OF_CMD	13
 static const char cmdflags[] = { 'I', 'D', 'D', 'R', 'A', 'L', 'F', 'Z',
-				 'N', 'X', 'P', 'C', 'E' };
+				 'N', 'X', 'P', 'E' };
 
 #define OPTION_OFFSET 256
 
@@ -110,7 +117,6 @@ static struct option original_opts[] = {
 	{ "list", 2, 0,  'L' },
 	{ "flush", 2, 0,  'F' },
 	{ "zero", 2, 0,  'Z' },
-	{ "check", 1, 0,  'C' },
 	{ "new-chain", 1, 0,  'N' },
 	{ "delete-chain", 2, 0,  'X' },
 	{ "rename-chain", 2, 0,  'E' },
@@ -140,7 +146,7 @@ static struct option original_opts[] = {
 struct ip6t_entry_target *
 ip6t_get_target(struct ip6t_entry *e)
 {
-	        return (void *)e + e->target_offset;
+	return (void *)e + e->target_offset;
 }
 #endif
 
@@ -284,7 +290,7 @@ exit_printhelp(void)
 	struct ip6tables_target *t = NULL;
 
 	printf("%s v%s\n\n"
-"Usage: %s -[ADC] chain rule-specification [options]\n"
+"Usage: %s -[AD] chain rule-specification [options]\n"
 "       %s -[RI] chain rulenum rule-specification [options]\n"
 "       %s -D chain rulenum [options]\n"
 "       %s -[LFZ] [chain] [options]\n"
@@ -310,7 +316,6 @@ exit_printhelp(void)
 "  --list    -L [chain]		List the rules in a chain or all chains\n"
 "  --flush   -F [chain]		Delete all rules in  chain or all chains\n"
 "  --zero    -Z [chain]		Zero counters in chain or all chains\n"
-"  --check   -C chain		Test this packet on chain\n"
 "  --new     -N chain		Create a new user-defined chain\n"
 "  --delete-chain\n"
 "            -X [chain]		Delete a user-defined chain\n"
@@ -740,12 +745,15 @@ find_match(const char *name, enum ip6t_tryload tryload)
 		else
 			ptr = NULL;
 	}
+	if(!ptr && (tryload == LOAD_MUST_SUCCEED)) {
+		exit_error(PARAMETER_PROBLEM,
+			   "Couldn't find match `%s'\n", name);
+	}
 #endif
 
 	if (ptr)
 		ptr->used = 1;
 
-
 	return ptr;
 }
 
@@ -952,6 +960,10 @@ find_target(const char *name, enum ip6t_tryload tryload)
 		else
 			ptr = NULL;
 	}
+	if(!ptr && (tryload == LOAD_MUST_SUCCEED)) {
+		exit_error(PARAMETER_PROBLEM,
+			   "Couldn't find target `%s'\n", name);
+	}
 #endif
 
 	if (ptr)
@@ -1153,9 +1165,6 @@ print_firewall(const struct ip6t_entry *fw,
 	u_int8_t flags;
 	char buf[BUFSIZ];
 
-	/* User creates a chain called "REJECT": this overrides the
-	   `REJECT' target module.  Keep feeding them rope until the
-	   revolution... Bwahahahahah */
 	if (!ip6tc_is_chain(targname, handle))
 		target = find_target(targname, TRY_LOAD);
 	else
@@ -1412,36 +1421,6 @@ delete_entry(const ip6t_chainlabel chain,
 	return ret;
 }
 
-static int
-check_packet(const ip6t_chainlabel chain,
-	     struct ip6t_entry *fw,
-	     unsigned int nsaddrs,
-	     const struct in6_addr saddrs[],
-	     unsigned int ndaddrs,
-	     const struct in6_addr daddrs[],
-	     int verbose,
-	     ip6tc_handle_t *handle)
-{
-	int ret = 1;
-	unsigned int i, j;
-	const char *msg;
-
-	for (i = 0; i < nsaddrs; i++) {
-		fw->ipv6.src = saddrs[i];
-		for (j = 0; j < ndaddrs; j++) {
-			fw->ipv6.dst = daddrs[j];
-			if (verbose)
-				print_firewall_line(fw, *handle);
-			msg = ip6tc_check_packet(chain, fw, handle);
-			if (!msg) ret = 0;
-			else printf("%s\n", msg);
-		}
-	}
-
-	return ret;
-}
-
-/*static int*/
 int
 for_each_chain(int (*fn)(const ip6t_chainlabel, int, ip6tc_handle_t *),
 	       int verbose, int builtinstoo, ip6tc_handle_t *handle)
@@ -1681,6 +1660,7 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 	const char *jumpto = "";
 	char *protocol = NULL;
 	const char *modprobe = NULL;
+	int proto_used = 0;
 	char icmp6p[] = "icmpv6";
 
 	memset(&fw, 0, sizeof(fw));
@@ -1709,7 +1689,7 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 	opterr = 0;
 
 	while ((c = getopt_long(argc, argv,
-	   "-A:C:D:R:I:L::F::Z::N:X::E:P:Vh::o:p:s:d:j:i:bvnt:m:xc:",
+	   "-A:D:R:I:L::M:F::Z::N:X::E:P:Vh::o:p:s:d:j:i:bvnt:m:xc:",
 					   opts, NULL)) != -1) {
 		switch (c) {
 			/*
@@ -1732,12 +1712,6 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 			}
 			break;
 
-		case 'C':
-			add_command(&command, CMD_CHECK, CMD_NONE,
-				    invert);
-			chain = optarg;
-			break;
-
 		case 'R':
 			add_command(&command, CMD_REPLACE, CMD_NONE,
 				    invert);
@@ -1789,6 +1763,14 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 			break;
 
 		case 'N':
+			if (optarg && *optarg == '-')
+				exit_error(PARAMETER_PROBLEM,
+					   "chain name not allowed to start "
+					   "with `-'\n");
+			if (find_target(optarg, TRY_LOAD))
+				exit_error(PARAMETER_PROBLEM,
+					   "chain name may not clash "
+					   "with target name\n");
 			add_command(&command, CMD_NEW_CHAIN, CMD_NONE,
 				    invert);
 			chain = optarg;
@@ -2044,14 +2026,41 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 
 				/* If you listen carefully, you can
 				   actually hear this code suck. */
+
+				/* some explanations (after four different bugs
+				 * in 3 different releases): If we encountere a
+				 * parameter, that has not been parsed yet,
+				 * it's not an option of an explicitly loaded
+				 * match or a target.  However, we support
+				 * implicit loading of the protocol match
+				 * extension.  '-p tcp' means 'l4 proto 6' and
+				 * at the same time 'load tcp protocol match on
+				 * demand if we specify --dport'.
+				 *
+				 * To make this work, we need to make sure:
+				 * - the parameter has not been parsed by
+				 *   a match (m above)
+				 * - a protocol has been specified
+				 * - the protocol extension has not been
+				 *   loaded yet, or is loaded and unused
+				 *   [think of iptables-restore!]
+				 * - the protocol extension can be successively
+				 *   loaded
+				 */
 				if (m == NULL
 				    && protocol
-				    && !find_proto(protocol, DONT_LOAD,
-						   options&OPT_NUMERIC)
+				    && (!find_proto(protocol, DONT_LOAD,
+						   options&OPT_NUMERIC) 
+					|| (find_proto(protocol, DONT_LOAD,
+							options&OPT_NUMERIC)
+					    && (proto_used == 0))
+				       )
 				    && (m = find_proto(protocol, TRY_LOAD,
 						       options&OPT_NUMERIC))) {
 					/* Try loading protocol */
 					size_t size;
+					
+					proto_used = 1;
 
 					size = IP6T_ALIGN(sizeof(struct ip6t_entry_match))
 							 + m->size;
@@ -2067,6 +2076,7 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 					optind--;
 					continue;
 				}
+
 				if (!m)
 					exit_error(PARAMETER_PROBLEM,
 						   "Unknown arg `%s'",
@@ -2097,8 +2107,7 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 		exit_error(PARAMETER_PROBLEM,
 			   "nothing appropriate following !");
 
-	if (command & (CMD_REPLACE | CMD_INSERT | CMD_DELETE | CMD_APPEND |
-	    CMD_CHECK)) {
+	if (command & (CMD_REPLACE | CMD_INSERT | CMD_DELETE | CMD_APPEND)) {
 		if (!(options & OPT_DESTINATION))
 			dhostnetworkmask = "::0/0";
 		if (!(options & OPT_SOURCE))
@@ -2118,10 +2127,6 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 		exit_error(PARAMETER_PROBLEM, "! not allowed with multiple"
 			   " source or destination IP addresses");
 
-	if (command == CMD_CHECK && fw.ipv6.invflags != 0)
-		exit_error(PARAMETER_PROBLEM, "! not allowed with -%c",
-			   cmd2char(CMD_CHECK));
-
 	if (command == CMD_REPLACE && (nsaddrs != 1 || ndaddrs != 1))
 		exit_error(PARAMETER_PROBLEM, "Replacement rule does not "
 			   "specify a unique address");
@@ -2148,8 +2153,7 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 			"can't initialize ip6tables table `%s': %s",
 			*table, ip6tc_strerror(errno));
 
-	if (command == CMD_CHECK
-	    || command == CMD_APPEND
+	if (command == CMD_APPEND
 	    || command == CMD_DELETE
 	    || command == CMD_INSERT
 	    || command == CMD_REPLACE) {
@@ -2161,12 +2165,6 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 					   "Can't use -%c with %s\n",
 					   opt2char(OPT_VIANAMEOUT),
 					   chain);
-			/* -i required with -C */
-			if (command == CMD_CHECK && !(options & OPT_VIANAMEIN))
-				exit_error(PARAMETER_PROBLEM,
-					   "Need -%c with %s\n",
-					   opt2char(OPT_VIANAMEIN),
-					   chain);
 		}
 
 		if (strcmp(chain, "POSTROUTING") == 0
@@ -2177,12 +2175,6 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 					   "Can't use -%c with %s\n",
 					   opt2char(OPT_VIANAMEIN),
 					   chain);
-			/* -o required with -C */
-			if (command == CMD_CHECK && !(options&OPT_VIANAMEOUT))
-				exit_error(PARAMETER_PROBLEM,
-					   "Need -%c with %s\n",
-					   opt2char(OPT_VIANAMEOUT),
-					   chain);
 		}
 
 		if (target && ip6tc_is_chain(jumpto, *handle)) {
@@ -2228,11 +2220,6 @@ int do_command6(int argc, char *argv[], char **table, ip6tc_handle_t *handle)
 				   options&OPT_VERBOSE,
 				   handle);
 		break;
-	case CMD_CHECK:
-		ret = check_packet(chain, e,
-				   nsaddrs, saddrs, ndaddrs, daddrs,
-				   options&OPT_VERBOSE, handle);
-		break;
 	case CMD_DELETE:
 		ret = delete_entry(chain, e,
 				   nsaddrs, saddrs, ndaddrs, daddrs,
