@@ -3,6 +3,13 @@
 /*
  * Author: Paul.Russell@rustcorp.com.au and mneuling@radlogic.com.au
  *
+ * (C) 2000-2002 by the netfilter coreteam <coreteam@netfilter.org>:
+ * 		    Paul 'Rusty' Russell <rusty@rustcorp.com.au>
+ * 		    Marc Boucher <marc+nf@mbsi.ca>
+ * 		    James Morris <jmorris@intercode.com.au>
+ * 		    Harald Welte <laforge@gnumonks.org>
+ * 		    Jozsef Kadlecsik <kadlec@blackhole.kfki.hu>
+ *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU General Public License as published by
  *	the Free Software Foundation; either version 2 of the License, or
@@ -79,7 +86,7 @@
 #define CMD_RENAME_CHAIN	0x1000U
 #define NUMBER_OF_CMD	13
 static const char cmdflags[] = { 'I', 'D', 'D', 'R', 'A', 'L', 'F', 'Z',
-				 'N', 'X', 'P', 'C', 'E' };
+				 'N', 'X', 'P', 'E' };
 
 #define OPTION_OFFSET 256
 
@@ -108,7 +115,6 @@ static struct option original_opts[] = {
 	{ "list", 2, 0,  'L' },
 	{ "flush", 2, 0,  'F' },
 	{ "zero", 2, 0,  'Z' },
-	{ "check", 1, 0,  'C' },
 	{ "new-chain", 1, 0,  'N' },
 	{ "delete-chain", 2, 0,  'X' },
 	{ "rename-chain", 2, 0,  'E' },
@@ -362,7 +368,6 @@ exit_printhelp(void)
 "  --list    -L [chain]		List the rules in a chain or all chains\n"
 "  --flush   -F [chain]		Delete all rules in  chain or all chains\n"
 "  --zero    -Z [chain]		Zero counters in chain or all chains\n"
-"  --check   -C chain		Test this packet on chain\n"
 "  --new     -N chain		Create a new user-defined chain\n"
 "  --delete-chain\n"
 "            -X [chain]		Delete a user-defined chain\n"
@@ -476,14 +481,20 @@ add_command(int *cmd, const int newcmd, const int othercmds, int invert)
 }
 
 int
-check_inverse(const char option[], int *invert)
+check_inverse(const char option[], int *invert, int *optind, int argc)
 {
 	if (option && strcmp(option, "!") == 0) {
 		if (*invert)
 			exit_error(PARAMETER_PROBLEM,
 				   "Multiple `!' flags not allowed");
-
 		*invert = TRUE;
+		if (optind) {
+			*optind = *optind+1;
+			if (argc && *optind > argc)
+				exit_error(PARAMETER_PROBLEM,
+					   "no argument following `!'");
+		}
+
 		return TRUE;
 	}
 	return FALSE;
@@ -604,7 +615,7 @@ parse_mask(char *mask)
 	return &maskaddr;
 }
 
-static void
+void
 parse_hostnetworkmask(const char *name, struct in_addr **addrpp,
 		      struct in_addr *maskp, unsigned int *naddrs)
 {
@@ -675,6 +686,10 @@ find_match(const char *name, enum ipt_tryload tryload)
 		else
 			ptr = NULL;
 	}
+	if(!ptr && (tryload == LOAD_MUST_SUCCEED)) {
+		exit_error(PARAMETER_PROBLEM,
+			   "Couldn't find match `%s'\n", name);
+	}
 #endif
 
 	if (ptr)
@@ -689,13 +704,18 @@ find_proto(const char *pname, enum ipt_tryload tryload, int nolookup)
 {
 	unsigned int proto;
 
-	if (string_to_number(pname, 0, 255, &proto) != -1)
-		return find_match(proto_to_name(proto, nolookup), tryload);
+	if (string_to_number(pname, 0, 255, &proto) != -1) {
+		char *protoname = proto_to_name(proto, nolookup);
 
-	return find_match(pname, tryload);
+		if (protoname)
+			return find_match(protoname, tryload);
+	} else
+		return find_match(pname, tryload);
+
+	return NULL;
 }
 
-static u_int16_t
+u_int16_t
 parse_protocol(const char *s)
 {
 	unsigned int proto;
@@ -751,7 +771,9 @@ parse_interface(const char *arg, char *vianame, unsigned char *mask)
 		memset(mask, 0xFF, vialen + 1);
 		memset(mask + vialen + 1, 0, IFNAMSIZ - vialen - 1);
 		for (i = 0; vianame[i]; i++) {
-			if (!isalnum(vianame[i]) && vianame[i] != '_') {
+			if (!isalnum(vianame[i]) 
+			    && vianame[i] != '_' 
+			    && vianame[i] != '.') {
 				printf("Warning: wierd character in interface"
 				       " `%s' (No aliases, :, ! or *).\n",
 				       vianame);
@@ -816,7 +838,8 @@ addr_to_dotted(const struct in_addr *addrp)
 	sprintf(buf, "%d.%d.%d.%d", bytep[0], bytep[1], bytep[2], bytep[3]);
 	return buf;
 }
-static char *
+
+char *
 addr_to_anyname(const struct in_addr *addr)
 {
 	char *name;
@@ -828,7 +851,7 @@ addr_to_anyname(const struct in_addr *addr)
 	return addr_to_dotted(addr);
 }
 
-static char *
+char *
 mask_to_dotted(const struct in_addr *mask)
 {
 	int i;
@@ -938,6 +961,10 @@ find_target(const char *name, enum ipt_tryload tryload)
 		else
 			ptr = NULL;
 	}
+	if(!ptr && (tryload == LOAD_MUST_SUCCEED)) {
+		exit_error(PARAMETER_PROBLEM,
+			   "Couldn't find target `%s'\n", name);
+	}
 #endif
 
 	if (ptr)
@@ -1139,9 +1166,6 @@ print_firewall(const struct ipt_entry *fw,
 	u_int8_t flags;
 	char buf[BUFSIZ];
 
-	/* User creates a chain called "REJECT": this overrides the
-	   `REJECT' target module.  Keep feeding them rope until the
-	   revolution... Bwahahahahah */
 	if (!iptc_is_chain(targname, handle))
 		target = find_target(targname, TRY_LOAD);
 	else
@@ -1396,35 +1420,6 @@ delete_entry(const ipt_chainlabel chain,
 	return ret;
 }
 
-static int
-check_packet(const ipt_chainlabel chain,
-	     struct ipt_entry *fw,
-	     unsigned int nsaddrs,
-	     const struct in_addr saddrs[],
-	     unsigned int ndaddrs,
-	     const struct in_addr daddrs[],
-	     int verbose,
-	     iptc_handle_t *handle)
-{
-	int ret = 1;
-	unsigned int i, j;
-	const char *msg;
-
-	for (i = 0; i < nsaddrs; i++) {
-		fw->ip.src.s_addr = saddrs[i].s_addr;
-		for (j = 0; j < ndaddrs; j++) {
-			fw->ip.dst.s_addr = daddrs[j].s_addr;
-			if (verbose)
-				print_firewall_line(fw, *handle);
-			msg = iptc_check_packet(chain, fw, handle);
-			if (!msg) ret = 0;
-			else printf("%s\n", msg);
-		}
-	}
-
-	return ret;
-}
-
 int
 for_each_chain(int (*fn)(const ipt_chainlabel, int, iptc_handle_t *),
 	       int verbose, int builtinstoo, iptc_handle_t *handle)
@@ -1691,7 +1686,7 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 	opterr = 0;
 
 	while ((c = getopt_long(argc, argv,
-	   "-A:C:D:R:I:L::F::Z::N:X::E:P:Vh::o:p:s:d:j:i:fbvnt:m:xc:",
+	   "-A:C:D:R:I:L::M:F::Z::N:X::E:P:Vh::o:p:s:d:j:i:fbvnt:m:xc:",
 					   opts, NULL)) != -1) {
 		switch (c) {
 			/*
@@ -1714,12 +1709,6 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 			}
 			break;
 
-		case 'C':
-			add_command(&command, CMD_CHECK, CMD_NONE,
-				    invert);
-			chain = optarg;
-			break;
-
 		case 'R':
 			add_command(&command, CMD_REPLACE, CMD_NONE,
 				    invert);
@@ -1771,6 +1760,14 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 			break;
 
 		case 'N':
+			if (optarg && *optarg == '-')
+				exit_error(PARAMETER_PROBLEM,
+					   "chain name not allowed to start "
+					   "with `-'\n");
+			if (find_target(optarg, TRY_LOAD))
+				exit_error(PARAMETER_PROBLEM,
+					   "chain name may not clash "
+					   "with target name\n");
 			add_command(&command, CMD_NEW_CHAIN, CMD_NONE,
 				    invert);
 			chain = optarg;
@@ -1826,8 +1823,7 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 			 * Option selection
 			 */
 		case 'p':
-			if (check_inverse(optarg, &invert))
-				optind++;
+			check_inverse(optarg, &invert, &optind, argc);
 			set_option(&options, OPT_PROTOCOL, &fw.ip.invflags,
 				   invert);
 
@@ -1843,11 +1839,24 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 				exit_error(PARAMETER_PROBLEM,
 					   "rule would never match protocol");
 			fw.nfcache |= NFC_IP_PROTO;
+
+			/* try to load match with protocol name */
+			if ((m = find_proto(protocol, TRY_LOAD,
+					    options&OPT_NUMERIC))) {
+				size_t size;
+				size = IPT_ALIGN(sizeof(struct ipt_entry_match))
+						 + m->size;
+				m->m = fw_calloc(1, size);
+				m->m->u.match_size = size;
+				strcpy(m->m->u.user.name, m->name);
+				m->init(m->m, &fw.nfcache);
+				opts = merge_options(opts, m->extra_opts, &m->option_offset);
+			}
+
 			break;
 
 		case 's':
-			if (check_inverse(optarg, &invert))
-				optind++;
+			check_inverse(optarg, &invert, &optind, argc);
 			set_option(&options, OPT_SOURCE, &fw.ip.invflags,
 				   invert);
 			shostnetworkmask = argv[optind-1];
@@ -1855,8 +1864,7 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 			break;
 
 		case 'd':
-			if (check_inverse(optarg, &invert))
-				optind++;
+			check_inverse(optarg, &invert, &optind, argc);
 			set_option(&options, OPT_DESTINATION, &fw.ip.invflags,
 				   invert);
 			dhostnetworkmask = argv[optind-1];
@@ -1886,8 +1894,7 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 
 
 		case 'i':
-			if (check_inverse(optarg, &invert))
-				optind++;
+			check_inverse(optarg, &invert, &optind, argc);
 			set_option(&options, OPT_VIANAMEIN, &fw.ip.invflags,
 				   invert);
 			parse_interface(argv[optind-1],
@@ -1897,8 +1904,7 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 			break;
 
 		case 'o':
-			if (check_inverse(optarg, &invert))
-				optind++;
+			check_inverse(optarg, &invert, &optind, argc);
 			set_option(&options, OPT_VIANAMEOUT, &fw.ip.invflags,
 				   invert);
 			parse_interface(argv[optind-1],
@@ -2032,32 +2038,6 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 						     &m->m))
 						break;
 				}
-
-				/* If you listen carefully, you can
-				   actually hear this code suck. */
-				if (m == NULL
-				    && protocol
-				    && !find_proto(protocol, DONT_LOAD,
-						   options&OPT_NUMERIC)
-				    && (m = find_proto(protocol, TRY_LOAD,
-						       options&OPT_NUMERIC))) {
-					/* Try loading protocol */
-					size_t size;
-
-					size = IPT_ALIGN(sizeof(struct ipt_entry_match))
-							 + m->size;
-
-					m->m = fw_calloc(1, size);
-					m->m->u.match_size = size;
-					strcpy(m->m->u.user.name, m->name);
-					m->init(m->m, &fw.nfcache);
-
-					opts = merge_options(opts,
-					    m->extra_opts, &m->option_offset);
-
-					optind--;
-					continue;
-				}
 				if (!m)
 					exit_error(PARAMETER_PROBLEM,
 						   "Unknown arg `%s'",
@@ -2088,8 +2068,7 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 		exit_error(PARAMETER_PROBLEM,
 			   "nothing appropriate following !");
 
-	if (command & (CMD_REPLACE | CMD_INSERT | CMD_DELETE | CMD_APPEND |
-	    CMD_CHECK)) {
+	if (command & (CMD_REPLACE | CMD_INSERT | CMD_DELETE | CMD_APPEND)) {
 		if (!(options & OPT_DESTINATION))
 			dhostnetworkmask = "0.0.0.0/0";
 		if (!(options & OPT_SOURCE))
@@ -2109,10 +2088,6 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 		exit_error(PARAMETER_PROBLEM, "! not allowed with multiple"
 			   " source or destination IP addresses");
 
-	if (command == CMD_CHECK && fw.ip.invflags != 0)
-		exit_error(PARAMETER_PROBLEM, "! not allowed with -%c",
-			   cmd2char(CMD_CHECK));
-
 	if (command == CMD_REPLACE && (nsaddrs != 1 || ndaddrs != 1))
 		exit_error(PARAMETER_PROBLEM, "Replacement rule does not "
 			   "specify a unique address");
@@ -2139,8 +2114,7 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 			   "can't initialize iptables table `%s': %s",
 			   *table, iptc_strerror(errno));
 
-	if (command == CMD_CHECK
-	    || command == CMD_APPEND
+	if (command == CMD_APPEND
 	    || command == CMD_DELETE
 	    || command == CMD_INSERT
 	    || command == CMD_REPLACE) {
@@ -2152,12 +2126,6 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 					   "Can't use -%c with %s\n",
 					   opt2char(OPT_VIANAMEOUT),
 					   chain);
-			/* -i required with -C */
-			if (command == CMD_CHECK && !(options & OPT_VIANAMEIN))
-				exit_error(PARAMETER_PROBLEM,
-					   "Need -%c with %s\n",
-					   opt2char(OPT_VIANAMEIN),
-					   chain);
 		}
 
 		if (strcmp(chain, "POSTROUTING") == 0
@@ -2168,12 +2136,6 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 					   "Can't use -%c with %s\n",
 					   opt2char(OPT_VIANAMEIN),
 					   chain);
-			/* -o required with -C */
-			if (command == CMD_CHECK && !(options&OPT_VIANAMEOUT))
-				exit_error(PARAMETER_PROBLEM,
-					   "Need -%c with %s\n",
-					   opt2char(OPT_VIANAMEOUT),
-					   chain);
 		}
 
 		if (target && iptc_is_chain(jumpto, *handle)) {
@@ -2219,11 +2181,6 @@ int do_command(int argc, char *argv[], char **table, iptc_handle_t *handle)
 				   options&OPT_VERBOSE,
 				   handle);
 		break;
-	case CMD_CHECK:
-		ret = check_packet(chain, e,
-				   nsaddrs, saddrs, ndaddrs, daddrs,
-				   options&OPT_VERBOSE, handle);
-		break;
 	case CMD_DELETE:
 		ret = delete_entry(chain, e,
 				   nsaddrs, saddrs, ndaddrs, daddrs,
